@@ -3,7 +3,7 @@
 use crate::{ser::Serializer, ArchiveUnsized, MetadataResolver, RelPtr, SerializeUnsized};
 use core::{
     borrow::Borrow,
-    cmp, hash,
+    cmp, fmt, hash,
     ops::{Deref, Index, RangeFull},
     pin::Pin,
 };
@@ -12,7 +12,6 @@ use std::ffi::CStr;
 /// An archived [`CString`](std::ffi::CString).
 ///
 /// Uses a [`RelPtr`] to a `CStr` under the hood.
-#[derive(Debug)]
 #[repr(transparent)]
 pub struct ArchivedCString(RelPtr<CStr>);
 
@@ -91,6 +90,13 @@ impl Borrow<CStr> for ArchivedCString {
     }
 }
 
+impl fmt::Debug for ArchivedCString {
+    #[inline]
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.as_c_str().fmt(f)
+    }
+}
+
 impl Deref for ArchivedCString {
     type Target = CStr;
 
